import os
from Bio import SeqIO
import argparse
import glob
import pandas as pd
import numpy as np



############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-t",
                    help="file name of your tree", type=str, default='16S.nwk',metavar='16S.nwk')
parser.add_argument("-n",
                    help="file name of your tree node names", type=str,
                    default='16S.format.name',metavar='16S.format.name')
parser.add_argument("-rd",
                    help="the reference data of gene traits", type=str,
                    default='Data.txt',metavar='Data.txt')
parser.add_argument("-a",
                    help="file name of your otu abundance", type=str,
                    default='abu.table',metavar='abu.table')
parser.add_argument("-r",
                    help="results_dir", type=str, default='ButyPhyl',
                    metavar='ButyPhyl')
parser.add_argument("-b",
                    help="dir to inferTraits tool", type=str,
                    default='inferTraits.py',
                    metavar='inferTraits.py')
parser.add_argument("--p",
                        help="further seperate the pathogens and commensals", type=str,
                        default='FALSE', metavar='FALSE or TRUE')
parser.add_argument("--sp",
                        help="further infer the species", type=str,
                        default='FALSE', metavar='FALSE or TRUE or your own reference species list')

################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.r)
except OSError:
    pass


################################################### Function ########################################################
def Traitspredictingspecies(filename, pathogen, speciesfile):
    # load traits
    Tempdf = pd.read_csv(filename, sep='\t',index_col=0,header=None)
    OTUwithTraits=dict()
    for OTUs in Tempdf.index:
        OTUwithTraits.setdefault(OTUs,float(Tempdf.loc[OTUs]))
    # set up species info
    Species = dict()
    Speciesall = []
    for lines in open(speciesfile,'r'):
        Species.setdefault(lines.split('\t')[0], lines.replace('\r','').replace('\n','').split('\t')[1:])
        for species in lines.replace('\r','').replace('\n','').split('\t')[1:]:
            if species not in Speciesall:
                Speciesall.append(species)
    if pathogen == 'FALSE':
        # load abu file
        OTU_table = pd.read_csv(args.a, sep='\t')
        Newrow = ['Abu_with_Traits']
        Newrow = Newrow + [0] * (len(OTU_table.columns) - 1)
        OTU_table.loc[-1] = Newrow
        OTU_table.set_index(OTU_table.columns[0], inplace=True)
        # set up species matrix
        Sprow = pd.DataFrame([[0] * (len(OTU_table.columns))], index=['species'],columns=OTU_table.columns)
        for species in Speciesall:
            Newrow = pd.DataFrame([[0] * (len(OTU_table.columns))], index=[species], columns=OTU_table.columns)
            Sprow=Sprow.append(Newrow)
        # set up butyrate producing species matrix
        Sprowbpb = pd.DataFrame([[0] * (len(OTU_table.columns))], index=['species'], columns=OTU_table.columns)
        for species in Speciesall:
            Newrow = pd.DataFrame([[0] * (len(OTU_table.columns))], index=[species], columns=OTU_table.columns)
            Sprowbpb=Sprowbpb.append(Newrow)
        # abundance times butyrate producing score
        for OTUs in OTU_table.index:
            try:
                # calculate species abundance
                # calculate bpb species abundance
                for species in Species[OTUs]:
                    if species == 'None':
                        # remove OTUs assigned to 2 species
                        OTUwithTraits[OTUs] = 0.0
                    #if OTUwithTraits[OTUs] == 0.5:
                        # reduce OTUs with uncertainty
                        #OTUwithTraits[OTUs] = 0.1
                    Sprow.loc[species] += OTU_table.loc[OTUs]
                    Sprowbpb.loc[species] += OTU_table.loc[OTUs] * OTUwithTraits[OTUs]
            except KeyError:
                pass
            try:
                OTU_table.loc[OTUs] = OTU_table.loc[OTUs] * OTUwithTraits[OTUs]
            except KeyError:
                # OTUs with low abundance that are filtered out
                OTU_table.loc[OTUs] = OTU_table.loc[OTUs] * 0.0
        # calculate total abundance of butyrate producing OTUs
        for OTUs in OTU_table.index:
            try:
                if OTUs != 'Abu_with_Traits':
                    OTU_table.loc['Abu_with_Traits'] += OTU_table.loc[OTUs]
            except KeyError:
                pass
        Newrow = OTU_table.loc['Abu_with_Traits']
        OTU_table[0:-1].to_csv(os.path.join(args.r, treefile + '.infertraits.otu_table'),
                                                         sep='\t', header=True)
        Newrow.to_csv(os.path.join(args.r, treefile + '.infertraits.abu'), sep='\t',
                                                           header=True)
        Sprow.to_csv(os.path.join(args.r, treefile + '.species.abu'), sep='\t',
                      header=True)
        Sprowbpb.to_csv(os.path.join(args.r, treefile + '.bpbspecies.abu'), sep='\t',
                      header=True)
    else:
        # load abu file
        OTU_table = pd.read_csv(args.a, sep='\t')
        Newrow = ['Abu_with_Traits']
        Newrow = Newrow + [0] * (len(OTU_table.columns) - 1)
        OTU_table.loc[-1] = Newrow
        OTU_table.set_index(OTU_table.columns[0], inplace=True)
        # load traits of pathogen
        Tempdfpathogen = pd.read_csv(pathogen, sep='\t', index_col=0, header=None)
        OTUwithPathogenTraits = dict()
        for OTUs in Tempdfpathogen.index:
            OTUwithPathogenTraits.setdefault(OTUs, float(Tempdfpathogen.loc[OTUs]))
        # load pathogen abu file
        OTU_table_pathogen = pd.read_csv(args.a, sep='\t')
        Newrow = ['Abu_with_Traits']
        Newrow = Newrow + [0] * (len(OTU_table_pathogen.columns) - 1)
        OTU_table_pathogen.loc[-1] = Newrow
        OTU_table_pathogen.set_index(OTU_table_pathogen.columns[0], inplace=True)
        # load commensal abu file
        OTU_table_commensal = pd.read_csv(args.a, sep='\t')
        Newrow = ['Abu_with_Traits']
        Newrow = Newrow + [0] * (len(OTU_table_commensal.columns) - 1)
        OTU_table_commensal.loc[-1] = Newrow
        OTU_table_commensal.set_index(OTU_table_commensal.columns[0], inplace=True)
        # set up species matrix
        Sprow = pd.DataFrame([[0] * (len(OTU_table.columns))], index=['species'], columns=OTU_table.columns)
        for species in Speciesall:
            Newrow = pd.DataFrame([[0] * (len(OTU_table.columns))], index=[species], columns=OTU_table.columns)
            Sprow = Sprow.append(Newrow)
        # set up pathogen butyrate producing species matrix
        Sprowbpb = pd.DataFrame([[0] * (len(OTU_table.columns))], index=['species'], columns=OTU_table.columns)
        for species in Speciesall:
            Newrow = pd.DataFrame([[0] * (len(OTU_table.columns))], index=[species], columns=OTU_table.columns)
            Sprowbpb = Sprowbpb.append(Newrow)
        # set up commensal butyrate producing species matrix
        Sprowbpbcom = pd.DataFrame([[0] * (len(OTU_table.columns))], index=['species'], columns=OTU_table.columns)
        for species in Speciesall:
            Newrow = pd.DataFrame([[0] * (len(OTU_table.columns))], index=[species], columns=OTU_table.columns)
            Sprowbpbcom = Sprowbpbcom.append(Newrow)
        # abundance times butyrate producing score
        for OTUs in OTU_table_pathogen.index:
            try:
                # calculate species abundance
                # calculate bpb species abundance
                for species in Species[OTUs]:
                    if species == 'None':
                        # remove OTUs assigned to 2 species
                        OTUwithTraits[OTUs] = 0.0
                    #if OTUwithTraits[OTUs] == 0.5:
                        # reduce OTUs with uncertainty
                        #OTUwithTraits[OTUs] = 0.1
                    Sprow.loc[species] = Sprow.loc[species] + OTU_table.loc[OTUs]
                    Sprowbpb.loc[species] = Sprowbpb.loc[species] + OTU_table_pathogen.loc[OTUs] * \
                                               OTUwithTraits[OTUs] * OTUwithPathogenTraits[OTUs]
                    Sprowbpbcom.loc[species] = Sprowbpbcom.loc[species] + OTU_table_commensal.loc[OTUs] * \
                                               OTUwithTraits[OTUs] * (1 - OTUwithPathogenTraits[OTUs])
            except KeyError:
                pass
            try:
                OTU_table_pathogen.loc[OTUs] = OTU_table_pathogen.loc[OTUs] * \
                                               OTUwithTraits[OTUs] * OTUwithPathogenTraits[OTUs]
                OTU_table_commensal.loc[OTUs] = OTU_table_commensal.loc[OTUs] * \
                                               OTUwithTraits[OTUs] * (1 - OTUwithPathogenTraits[OTUs])
            except KeyError:
                # OTUs with low abundance that are filtered out
                OTU_table_pathogen.loc[OTUs] = OTU_table_pathogen.loc[OTUs] * 0.0
                OTU_table_commensal.loc[OTUs] = OTU_table_commensal.loc[OTUs] * 0.0
        # calculate total abundance of butyrate producing OTUs
        for OTUs in OTU_table_pathogen.index:
            try:
                if OTUs != 'Abu_with_Traits':
                    OTU_table_pathogen.loc['Abu_with_Traits'] += OTU_table_pathogen.loc[OTUs]
                    OTU_table_commensal.loc['Abu_with_Traits'] += OTU_table_commensal.loc[OTUs]
            except KeyError:
                pass
        Newrow = OTU_table_pathogen.loc['Abu_with_Traits']
        OTU_table_pathogen[0:-1].to_csv(os.path.join(args.r, treefile + '.infertraits.pathogen.otu_table'),
                               sep='\t', header=True)
        Newrow.to_csv(os.path.join(args.r, treefile + '.infertraits.pathogen.abu'), sep='\t',
                      header=True)
        Newrow = OTU_table_commensal.loc['Abu_with_Traits']
        OTU_table_commensal[0:-1].to_csv(os.path.join(args.r, treefile + '.infertraits.commensal.otu_table'),
                               sep='\t', header=True)
        Newrow.to_csv(os.path.join(args.r, treefile + '.infertraits.commensal.abu'), sep='\t',
                      header=True)
        Sprow.to_csv(os.path.join(args.r, treefile + '.species.abu'), sep='\t',
                     header=True)
        Sprowbpb.to_csv(os.path.join(args.r, treefile + '.bpbspecies.pathogen.abu'), sep='\t',
                        header=True)
        Sprowbpbcom.to_csv(os.path.join(args.r, treefile + '.bpbspecies.commensal.abu'), sep='\t',
                        header=True)


def Traitspredicting(filename):
    Tempdf = pd.read_csv(filename, sep='\t',index_col=0,header=None)
    OTUwithTraits=dict()
    for OTUs in Tempdf.index:
        OTUwithTraits.setdefault(OTUs,float(Tempdf.loc[OTUs]))
    OTU_table = pd.read_csv(args.a, sep='\t')
    Newrow = ['Abu_with_Traits']
    Newrow = Newrow + [0]*(len(OTU_table.columns)-1)
    OTU_table.loc[-1] = Newrow
    OTU_table.set_index(OTU_table.columns[0],inplace=True)
    # abundance times butyrate producing score
    for OTUs in OTU_table.index:
        #if OTUwithTraits[OTUs] == 0.5:
            # reduce OTUs with uncertainty
            #OTUwithTraits[OTUs] = 0.1
        try:
            OTU_table.loc[OTUs] = OTU_table.loc[OTUs]*OTUwithTraits[OTUs]
        except KeyError:
            # OTUs with low abundance that are filtered out
            OTU_table.loc[OTUs] = OTU_table.loc[OTUs] * 0.0
    # calculate total abundance of butyrate producing OTUs
    for OTUs in OTU_table.index:
        try:
            if OTUs != 'Abu_with_Traits':
                OTU_table.loc['Abu_with_Traits'] += OTU_table.loc[OTUs]
        except KeyError:
            pass
    Newrow = OTU_table.loc['Abu_with_Traits']
    OTU_table[0:-1].to_csv(os.path.join(args.r, treefile + '.infertraits.otu_table'),
                                                     sep='\t', header=True)
    Newrow.to_csv(os.path.join(args.r, treefile + '.infertraits.abu'), sep='\t',
                                                       header=True)


################################################### Programme #######################################################
rootofus, treefile = os.path.split(args.t)
traitdir, traitfile = os.path.split(args.rd)
os.system('python '+args.b+' -t ' + str(args.t) + ' -n ' + \
           str(args.n)+' -rd ' + \
           str(args.rd) +' -r ' \
           + str(args.r) )
if args.p != 'FALSE':
    os.system(('python ' + args.b + ' -t ' + str(args.t) + ' -n ' + \
              str(args.n) + ' -rd ' + \
              str(args.p) + ' -r ' \
              + str(args.r) + ' -tag .pathogen'))
    os.system(('python ' + args.b + ' -t ' + str(args.t) + ' -n ' + \
               str(args.n) + ' -rd ' + \
               str(args.sp) + ' -r ' \
               + str(args.r)).replace('inferTraits.py', 'inferTraits_species.py'))
    Traitspredictingspecies(os.path.join(args.r, treefile + '.infertraits.txt'),
                     os.path.join(args.r, treefile + '.infertraits.pathogen.txt'),
                     os.path.join(args.r, treefile + '.infertraits.species.txt'))
elif args.sp !='FALSE':
    os.system(('python ' + args.b + ' -t ' + str(args.t) + ' -n ' + \
               str(args.n) + ' -rd ' + \
               str(args.sp) + ' -r ' \
               + str(args.r)).replace('inferTraits.py', 'inferTraits_species.py'))
    Traitspredictingspecies(os.path.join(args.r, treefile + '.infertraits.txt'),
                     args.p,
                     os.path.join(args.r, treefile + '.infertraits.species.txt'))
else:
    Traitspredicting(os.path.join(args.r,treefile+'.infertraits.txt'))




import os
from Bio import SeqIO
import argparse
import glob
import buty_phyl
import sys


################################################### Decalration #######################################################
print ("\
------------------------------------------------------------------------\n\
buty_phyl infers traits by 16S\n\
input: otu table (-t) and otu sequences (-s)\n\
requirement: mafft and fasttree\n\n\
Copyright:An Ni Zhang, Prof. Eric Alm, MIT\n\n\
Citation:\n\
Contact anniz44@mit.edu\n\
------------------------------------------------------------------------\n\
")

def main():
    usage = ("usage: buty_phyl -t your.otu.table -s your.otu.seqs")
    version_string = 'buty_phyl {v}, on Python {pyv[0]}.{pyv[1]}.{pyv[2]}'.format(
        v=buty_phyl.__version__,
        pyv=sys.version_info,
    )
    ############################################ Arguments and declarations ##############################################
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-t",
                        help="file name of your otu_table", type=str,
                        default='example/example.otu_table', metavar='your.otu.table')
    parser.add_argument("-s",
                        help="file name of your otu_seq", type=str,
                        default='your.otu.fasta', metavar='your.otu.fasta')
    parser.add_argument("-r",
                        help="results_dir", type=str, default='ButyPhyl', metavar='ButyPhyl')
    parser.add_argument("-top",
                        help="number of top OTUs", type=int, default=1000, metavar=1000)
    parser.add_argument("--m",
                        help="the path of mafft", type=str,
                        default='/usr/local/bin/mafft', metavar='mafft')
    parser.add_argument("--ft",
                        help="the path of fasttree", type=str,
                        default='/usr/local/bin/FastTree', metavar='your FastTree')
    parser.add_argument("--rs",
                        help="the reference 16S", type=str,
                        default='default', metavar='default or your own reference 16s sequences')
    parser.add_argument("--rt",
                        help="the reference data of gene traits", type=str,
                        default='b', metavar='b (for butyrate), or n (for nitrate), or s (for sulfate),' +
                                                    ' or your own reference traits')
    parser.add_argument("--th",
                        help="number of threads", type=int, default=1, metavar=1)
    parser.add_argument("--test",
                        help="test the buty_phyl", action="store_true")
    parser.add_argument("--p",
                        help="further seperate the pathogens and commensals", type=str,
                        default='FALSE', metavar='FALSE or TRUE or your own reference pathogen list')
    parser.add_argument("--sp",
                        help="further infer the species", type=str,
                        default='FALSE', metavar='FALSE or TRUE or your own reference species list')
    ################################################## Definition ########################################################
    args = parser.parse_args()
    workingdir=os.path.abspath(os.path.dirname(__file__))
    ref_pathogen = 'FALSE'
    ref_sp = 'FALSE'
    if args.rs == 'default':
        ref_tree = os.path.join(workingdir,'data/Complete_genome_16S.fasta')
    else:
        ref_tree = args.rs
    if args.rt == 'b':
        ref_traits = os.path.join(workingdir, 'data/GMC_CG_buk_ptbORbut.txt')
        ref_tree = os.path.join(workingdir, 'data/GMC_CG_16S.fasta')
        if args.sp == 'TRUE':
            ref_sp = os.path.join(workingdir, 'data/GMC_CG_species.txt')
        else:
            ref_sp = args.sp
        if args.p == 'TRUE':
            ref_pathogen = os.path.join(workingdir, 'data/GMC_CG_pathogen.txt')
            if args.sp == 'FALSE':
                ref_sp = os.path.join(workingdir, 'data/GMC_CG_species.txt')
        else:
            ref_pathogen = args.p
    elif args.rt == 'n':
        ref_traits = os.path.join(workingdir,'data/Genome_NR.txt')
    elif args.rt == 's':
        ref_traits = os.path.join(workingdir, 'data/Genome_SRB.txt')
    else:
        ref_traits = args.rt
    if args.test:
        input_table = os.path.join(workingdir, 'example/example.otu_table')
        input_seq = os.path.join(workingdir, 'example/example.otu_seqs')
    else:
        if args.t == 'example/example.otu_table':
            print('testing buty_phyl')
            input_table = os.path.join(workingdir, 'example/example.otu_table')
            input_seq = os.path.join(workingdir, 'example/example.otu_seqs')
        else:
            input_table = args.t
            input_seq = args.s
    #result_dir = args.r + '_' + os.path.split(ref_traits)[-1]
    result_dir = args.r
    try:
        os.mkdir(result_dir)
    except OSError:
        pass
    try:
        os.mkdir(result_dir + '/Bayers_model')
    except OSError:
        pass
    rootofus, otuseq = os.path.split(input_seq)
    rootofutb, otutable = os.path.split(input_table)
    ################################################### Programme #######################################################
    f1 = open(os.path.join(result_dir, otutable + 'BayersTraits.log'), 'w')
    # filter the OTU sequences of top args.top max abudance
    print ('filter the OTU sequences of top ' + str(args.top) + ' max abudance')
    try:
        ftry = open(str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter', 'r')
    except IOError:
        cmd = 'python ' + workingdir + '/scripts/OTU.filter.py -t ' + str(input_table) + ' -s ' + str(input_seq) \
               + ' -top ' + str(args.top) + ' -r ' + str(result_dir) + '/Filtered_OTU \n'
        os.system(cmd)
        f1.write(cmd)
    # align the otus with reference 16S
    print('align the otus sequences with reference 16S sequences\nit takes quite a while...')
    try:
        ftry = open(str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter.align.16S', 'r')
    except IOError:
        cmd = 'cat ' + str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter ' + str(ref_tree) + ' > ' + \
               str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter.16S \n'
        os.system(cmd)
        f1.write(cmd)
        # InferTraits
        cmd = args.m + ' --nuc --adjustdirection --quiet --maxiterate 0 --retree 2 --nofft --thread ' + str(
            args.th) + ' ' + \
               str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter.16S > ' + \
               str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter.align.16S \n'
        os.system(cmd)
        f1.write(cmd)
    print('alignment finished!\nnow we are building the tree\nit also takes quite a while...')
    try:
        ftry = open(str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter.align.16S.nwk', 'r')
    except IOError:
        cmd = 'python ' + workingdir + '/scripts/treeformat.py -a ' + str(result_dir) + \
               '/Filtered_OTU/' + str(otuseq) + '.filter.align.16S \n'
        os.system(cmd)
        f1.write(cmd)
        cmd = args.ft + ' -quiet -fastest -nt -nosupport ' + str(result_dir) + '/Filtered_OTU/' + \
               str(otuseq) + '.filter.align.16S.format > ' + \
               str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter.align.16S.nwk \n'
        os.system(cmd)
        f1.write(cmd)
    # run BayersTraits
    # build the model
    print('infer the traits based on 16s\nwe are almost there!')
    try:
        ftry = open(str(result_dir) + '/Bayers_model/' + str(otuseq) + '.filter.align.16S.nwk.infertraits.txt', 'r')
    except IOError:
        # InferTraits only
        if ref_sp == 'FALSE':
            cmd = 'python ' + workingdir + '/scripts/Bayers.model.py -t ' + str(result_dir) + '/Filtered_OTU/' + str(
                otuseq) + '.filter.align.16S.nwk -n ' + \
                  str(result_dir) + '/Filtered_OTU/' + str(otuseq) + '.filter.align.16S.format.name -rd ' + \
                  str(ref_traits) + ' -a ' + \
                  str(result_dir) + '/Filtered_OTU/' + str(otutable) + '.abu.table -r ' \
                  + str(result_dir) + '/Bayers_model -b ' + str(workingdir + "/scripts/inferTraits.py") + \
                  ' --p '+str(ref_pathogen)+' \n'
        else:
            cmd = 'python ' + workingdir + '/scripts/Bayers.model.species.py -t ' + str(args.r) + '/Filtered_OTU/' + str(
                otuseq) + '.filter.align.16S.nwk -n ' + \
                  str(args.r) + '/Filtered_OTU/' + str(otuseq) + '.filter.align.16S.format.name -rd ' + \
                  str(ref_traits) + ' -a ' + \
                  str(args.r) + '/Filtered_OTU/' + str(otutable) + '.abu.table -r ' \
                  + str(args.r) + '/Bayers_model -b ' + str(workingdir + "/scripts/inferTraits.py") + \
                  ' --p ' + str(ref_pathogen) + ' --sp ' + str(ref_sp) + ' \n'
        os.system(cmd)
        f1.write(cmd)
    f1.close()

################################################## Function ########################################################

if __name__ == '__main__':
    main()



#!/Users/ericalm/anaconda/envs/py36/bin/python
import argparse
import os
import random
import math
import csv
import re
import sys
import pandas as pd
import numpy as np
from Bio import SeqIO
from Bio.Blast.Applications import NcbiblastnCommandline
from Bio import SearchIO
from Bio import Phylo
from functools import reduce


############################################ Arguments and declarations ##############################################
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument("-t",
                    help="file name of your tree", type=str, default='16S.nwk',metavar='16S.nwk')
parser.add_argument("-n",
                    help="file name of your tree node names", type=str, default='16S.format.name',metavar='16S.format.name')
parser.add_argument("-rd",
                    help="the reference data of gene traits", type=str, default='Data.txt',metavar='Data.txt')
parser.add_argument("-r",
                    help="results_dir", type=str, default='ButyPhyl',metavar='ButyPhyl')

################################################## Definition ########################################################
args = parser.parse_args()
try:
    os.mkdir(args.r)
except OSError:
    pass


################################################### Function ########################################################
def read_table(filename,reverse=0):
    table = {}
    f = open(filename,'r')
    lines = [ x.rstrip("\n").split() for x in f ]
    if reverse:
        lines = [ (v,k) for (k,v) in lines ]
    for k,v in lines:
        table[k] = v
    f.close()
    return table

def assign_internal_names(tree):
    names = {}
    for idx, clade in enumerate(tree.find_clades()):
        if clade.name:
            old_name = clade.name
            clade.name = '%d_%s' % (idx, clade.name)
        else:
            old_name = None
            clade.name = str(idx)
        if old_name:
            names[old_name] = clade.name
    return names

def mismatch(c,chars):
    return len([x for x in chars if c not in x])

def pars(chars):
    # clade terminal
    if not chars:
        return set()
    # internal clades
    # chars is all scores of clades under the internal clades
    # all_chars is all unique stored anno/scores in chars
    all_chars = set(reduce( lambda a,b: a.union(b), chars ))
    # scores are [a score, number of different scores]
    scores = [(c,mismatch(c,chars)) for c in all_chars]
    # min_score is the minimum number of different scores
    min_score = min(scores,key=lambda x:x[1])[1]
    # choose the scores of highest agreement (minimum number of different scores)
    # we set 0.5 as the cutoff
    # return {0.0} for more 0.0 than 1.0
    # return {1.0} for more 1.0 than 0.0
    # return {0.0, 1.0} for half 0.0 and half 1.0 (final result is 0.5)
    return set([x[0] for x in scores if not x[1] > min_score])

def down_pass(clade,data,anno):
    # set the anno/scores from bottom to up
    for x in clade.clades:
        down_pass(x,data,anno)
    # set anno to reference terminal clades
    if clade.name in anno:
        data[clade.name] = set([str(anno[clade.name])])
        return
    # ignore unknown terminal clades
    if clade.is_terminal():
        return
    # infer traits for internal clades that have reference terminal clades below
    # based on all scores of clades under the internal clades
    # pass the scores of highest agreement (minimum number of different scores)
    chars = pars([ data[x.name] for x in clade.clades if x.name in data ])
    if chars:
        data[clade.name] = chars
    return

def up_pass(parent_chars,clade,data,anno):
    # set the anno/scores from up to bottom
    # reference terminal clades
    if clade.name in anno:
        data[clade.name] = set([str(anno[clade.name])])
        if clade.is_terminal():
            return
    else:
        # correct scores for internal clades using parent scores
        # based on the scores of parent and itself
        if clade.name in data:
            data[clade.name] = pars([data[clade.name],parent_chars])
        # infer traits for internal clades that have no reference terminal clade below
        # infer traits for unknown terminal clades
        # pass the scores of parent
        else:
            data[clade.name] = parent_chars
    for x in clade.clades:
        up_pass(data[clade.name],x,data,anno)
    return


################################################### Programme #######################################################
workingDir, filename = os.path.split(args.t)
treefile = args.t
nodenamefile = args.n
annotationfile = args.rd

# read annotation files of traits in references (0.0 or 1.0)
anno = read_table(annotationfile)
# get numericIDs for each name
nodename = read_table(nodenamefile,1)

tree = Phylo.read(treefile, 'newick')
# assign names to internal nodes
internal_names = assign_internal_names(tree)
# switch annotations to new names
anno = dict([ (internal_names[nodename[k]],v) for (k,v) in anno.items() ])

# data stores the scores of internal and terminal clades
data = {}
# set the scores of internal and reference terminal clades
down_pass(tree.clade,data,anno)
up_pass(set(),tree.clade,data,anno)

leaves = [ (x,data[x]) for x in data.keys() if '_' in x ]
old_names = dict([ (v,k) for (k,v) in nodename.items() ]) #123->abc.fasta
new_names = dict([ (v,k) for (k,v) in internal_names.items() ]) #789_123->123

missing = []
f1=open(os.path.join(args.r,filename+'.infertraits.species.txt'),'w')
# output the mean score of the scores with the highest agreement (0.0, 0.5 or 1.0)
for k,v in leaves:
    if new_names[k] in old_names and k not in anno:
        if len(list(v)) == 1:
            f1.write(old_names[new_names[k]]+'\t'
                     +str(v).replace('{','').replace('}','').replace('\'','')+'\n')
        elif len(list(v)) < 2:
            f1.write(old_names[new_names[k]]+'\t'+'\t'.join(list(v))+'\n')
        else:
            f1.write(old_names[new_names[k]] + '\tNone\n')
    else:
        missing += [new_names[k]]
f1.close()
#print('Warning missing nodes in tree: ', missing)



from setuptools import setup

setup(
    name="buty_phyl",
    packages=['buty_phyl'],
    version="1.0.3",
    description="Inferring traits by 16S",
    author='Anni Zhang',
    author_email='anniz44@mit.edu',
    url='https://github.com/caozhichongchong/BayersTraits_16S',
    keywords=['16S', 'bacterial genomes', 'function', 'traits'],
    license='MIT',
    #install_requires=['python>=3.0'],
    include_package_data=True,
    long_description=open('README.md').read(),
    package_dir={'buty_phyl': 'buty_phyl'},
    package_data={'buty_phyl': ['scripts/*','data/*','example/*']},
    entry_points={'console_scripts': ['buty_phyl = buty_phyl.__main__:main']},
    #zip_safe=False,
    #setup_requires=['pytest-runner'],
    #tests_require=['pytest'],
    classifiers=[
        #'Development Status :: 1 - Alpha',
        #'Intended Audience :: Bioinformatics and Researchers',
        #'License :: MIT',
        #'Operating System :: MacOS',
        #'Operating System :: Microsoft :: Windows',
        #'Operating System :: LINUX',
        'Programming Language :: Python :: 3',
        #'Topic :: Antibiotic resistance :: risk ranking',
        #'Topic :: Metagenomes :: Antibiotic resistance',
    ]
)
